@@ -252,8 +252,9 @@ def display(*objs, include=None, exclude=None, metadata=None, transient=None, di
     display in IPython by following the above approach. But in practice, you
     often need to work with existing classes that you can't easily modify.
 
-    You can refer to the documentation on IPython display formatters in order to
-    register custom formatters for already existing types.
+    You can refer to the documentation on integrating with the display system in
+    order to register custom formatters for already existing types
+    (:ref:`integrating_rich_display`).
 
     .. versionadded:: 5.4 display available without import
     .. versionadded:: 6.1 display available without import
@@ -774,6 +775,19 @@ class ProgressBar(DisplayObject):
         self._progress = value
         self.update()
 
+    def __iter__(self):
+        self.display()
+        self._progress = -1 # First iteration is 0
+        return self
+
+    def __next__(self):
+        """Returns current value and increments display by one."""
+        self.progress += 1
+        if self.progress < self.total:
+            return self.progress
+        else:
+            raise StopIteration()
+
 class JSON(DisplayObject):
     """JSON expects a JSON-able dict or list
 
@@ -1098,9 +1112,9 @@ class Image(DisplayObject):
             if ext is not None:
                 if ext == u'jpg' or ext == u'jpeg':
                     format = self._FMT_JPEG
-                if ext == u'png':
+                elif ext == u'png':
                     format = self._FMT_PNG
-                if ext == u'gif':
+                elif ext == u'gif':
                     format = self._FMT_GIF
                 else:
                     format = ext.lower()
