@@ -316,13 +316,17 @@ def test_jedi():
         start = start if start is not None else l
         end = end if end is not None else l
         with provisionalcompleter():
+            ip.Completer.use_jedi = True
             completions = set(ip.Completer.completions(s, l))
+            ip.Completer.use_jedi = False
             assert_in(Completion(start, end, comp), completions, reason)
 
     def _test_not_complete(reason, s, comp):
         l = len(s)
         with provisionalcompleter():
+            ip.Completer.use_jedi = True
             completions = set(ip.Completer.completions(s, l))
+            ip.Completer.use_jedi = False
             assert_not_in(Completion(l, l, comp), completions, reason)
 
     import jedi
@@ -341,8 +345,10 @@ def test_completion_have_signature():
     """
     ip = get_ipython()
     with provisionalcompleter():
+        ip.Completer.use_jedi = True
         completions = ip.Completer.completions('ope', 3)
         c = next(completions)  # should be `open`
+        ip.Completer.use_jedi = False
     assert 'file' in c.signature, "Signature of function was not found by completer"
     assert 'encoding' in c.signature, "Signature of function was not found by completer"
 
@@ -357,7 +363,9 @@ def test_deduplicate_completions():
         zoo = 1
     '''))
     with provisionalcompleter():
+        ip.Completer.use_jedi = True
         l = list(_deduplicate_completions('Z.z', ip.Completer.completions('Z.z', 3)))
+        ip.Completer.use_jedi = False
 
     assert len(l) == 1, 'Completions (Z.z<tab>) correctly deduplicate: %s ' % l
     assert l[0].text == 'zoo'  # and not `it.accumulate`
@@ -367,8 +375,8 @@ def test_greedy_completions():
     """
     Test the capability of the Greedy completer. 
 
-    Most of the test here do not really show off the greedy completer, for proof
-    each of the text bellow now pass with Jedi. The greedy completer is capable of more. 
+    Most of the test here does not really show off the greedy completer, for proof
+    each of the text below now pass with Jedi. The greedy completer is capable of more. 
 
     See the :any:`test_dict_key_completion_contexts`
 
@@ -380,10 +388,13 @@ def test_greedy_completions():
                     "Shouldn't have completed on a[0]: %s"%c)
     with greedy_completion(), provisionalcompleter():
         def _(line, cursor_pos, expect, message, completion):
+            ip.Completer.use_jedi = False
             _,c = ip.complete('.', line=line, cursor_pos=cursor_pos)
+            nt.assert_in(expect, c, message % c)
+
+            ip.Completer.use_jedi = True
             with provisionalcompleter():
                 completions = ip.Completer.completions(line, cursor_pos)
-            nt.assert_in(expect, c, message%c)
             nt.assert_in(completion, completions)
 
         yield _, 'a[0].', 5, 'a[0].real', "Should have completed on a[0].: %s", Completion(5,5, 'real')
@@ -404,13 +415,14 @@ def test_omit__names():
     cfg.IPCompleter.omit__names = 0
     c.update_config(cfg)
     with provisionalcompleter():
+        c.use_jedi = False
         s,matches = c.complete('ip.')
-        completions = set(c.completions('ip.', 3))
-
         nt.assert_in('ip.__str__', matches)
-        nt.assert_in(Completion(3, 3, '__str__'), completions)
-        
         nt.assert_in('ip._hidden_attr', matches)
+
+        c.use_jedi = True
+        completions = set(c.completions('ip.', 3))
+        nt.assert_in(Completion(3, 3, '__str__'), completions)
         nt.assert_in(Completion(3,3, "_hidden_attr"), completions)
 
 
@@ -418,33 +430,37 @@ def test_omit__names():
     cfg.IPCompleter.omit__names = 1
     c.update_config(cfg)
     with provisionalcompleter():
+        c.use_jedi = False
         s,matches = c.complete('ip.')
-        completions = set(c.completions('ip.', 3))
-
         nt.assert_not_in('ip.__str__', matches)
-        nt.assert_not_in(Completion(3,3,'__str__'), completions)
-
         # nt.assert_in('ip._hidden_attr', matches)
+
+        c.use_jedi = True
+        completions = set(c.completions('ip.', 3))
+        nt.assert_not_in(Completion(3,3,'__str__'), completions)
         nt.assert_in(Completion(3,3, "_hidden_attr"), completions)
 
     cfg = Config()
     cfg.IPCompleter.omit__names = 2
     c.update_config(cfg)
     with provisionalcompleter():
+        c.use_jedi = False
         s,matches = c.complete('ip.')
-        completions = set(c.completions('ip.', 3))
-
         nt.assert_not_in('ip.__str__', matches)
-        nt.assert_not_in(Completion(3,3,'__str__'), completions)
-
         nt.assert_not_in('ip._hidden_attr', matches)
+
+        c.use_jedi = True
+        completions = set(c.completions('ip.', 3))
+        nt.assert_not_in(Completion(3,3,'__str__'), completions)
         nt.assert_not_in(Completion(3,3, "_hidden_attr"), completions)
 
     with provisionalcompleter():
+        c.use_jedi = False
         s,matches = c.complete('ip._x.')
-        completions = set(c.completions('ip._x.', 6))
-
         nt.assert_in('ip._x.keys', matches)
+
+        c.use_jedi = True
+        completions = set(c.completions('ip._x.', 6))
         nt.assert_in(Completion(6,6, "keys"), completions)
 
     del ip._hidden_attr
@@ -457,6 +473,7 @@ def test_limit_to__all__False_ok():
     """
     ip = get_ipython()
     c = ip.Completer
+    c.use_jedi = False
     ip.ex('class D: x=24')
     ip.ex('d=D()')
     cfg = Config()
@@ -483,6 +500,7 @@ def test_get__all__entries_no__all__ok():
 def test_func_kw_completions():
     ip = get_ipython()
     c = ip.Completer
+    c.use_jedi = False
     ip.ex('def myfunc(a=1,b=2): return a+b')
     s, matches = c.complete(None, 'myfunc(1,b')
     nt.assert_in('b=', matches)
@@ -573,6 +591,7 @@ def test_magic_completion_order():
 def test_magic_completion_shadowing():
     ip = get_ipython()
     c = ip.Completer
+    c.use_jedi = False
 
     # Before importing matplotlib, %matplotlib magic should be the only option.
     text, matches = c.complete("mat")
@@ -939,6 +958,34 @@ def test_object_key_completion():
     nt.assert_in('qwick', matches)
 
 
+class NamedInstanceMetaclass(type):
+    def __getitem__(cls, item):
+        return cls.get_instance(item)
+
+class NamedInstanceClass(object, metaclass=NamedInstanceMetaclass):
+    def __init__(self, name):
+        if not hasattr(self.__class__, 'instances'):
+            self.__class__.instances = {}
+        self.__class__.instances[name] = self
+
+    @classmethod
+    def _ipython_key_completions_(cls):
+        return cls.instances.keys()
+
+    @classmethod
+    def get_instance(cls, name):
+        return cls.instances[name]
+
+def test_class_key_completion():
+    ip = get_ipython()
+    NamedInstanceClass('qwerty')
+    NamedInstanceClass('qwick')
+    ip.user_ns['named_instance_class'] = NamedInstanceClass
+
+    _, matches = ip.Completer.complete(line_buffer="named_instance_class['qw")
+    nt.assert_in('qwerty', matches)
+    nt.assert_in('qwick', matches)
+
 def test_tryimport():
     """
     Test that try-import don't crash on trailing dot, and import modules before
@@ -974,6 +1021,7 @@ def test_from_module_completer():
 
 def test_snake_case_completion():
     ip = get_ipython()
+    ip.Completer.use_jedi = False
     ip.user_ns['some_three'] = 3
     ip.user_ns['some_four'] = 4
     _, matches = ip.complete("s_", "print(s_f")
