@@ -1,5 +1,4 @@
 """Terminal input and output prompts."""
-from __future__ import print_function
 
 from pygments.token import Token
 import sys
@@ -63,12 +62,18 @@ class RichPromptDisplayHook(DisplayHook):
     """Subclass of base display hook using coloured prompt"""
     def write_output_prompt(self):
         sys.stdout.write(self.shell.separate_out)
-        self.prompt_end_newline = False
+        # If we're not displaying a prompt, it effectively ends with a newline,
+        # because the output will be left-aligned.
+        self.prompt_end_newline = True
+
         if self.do_full_cache:
             tokens = self.shell.prompts.out_prompt_tokens()
-            if tokens and tokens[-1][1].endswith('\n'):
-                self.prompt_end_newline = True
+            prompt_txt = ''.join(s for t, s in tokens)
+            if prompt_txt and not prompt_txt.endswith('\n'):
+                # Ask for a newline before multiline output
+                self.prompt_end_newline = False
+
             if self.shell.pt_cli:
                 self.shell.pt_cli.print_tokens(tokens)
             else:
-                print(*(s for t, s in tokens), sep='')
+                sys.stdout.write(prompt_txt)
