@@ -9,7 +9,6 @@
 #  the file COPYING, distributed as part of this software.
 #-----------------------------------------------------------------------------
 
-from .py3compat import xrange
 
 def uniq_stable(elems):
     """uniq_stable(elems) -> list
@@ -32,6 +31,6 @@ def flatten(seq):
 
 def chop(seq, size):
     """Chop a sequence into chunks of the given size."""
-    return [seq[i:i+size] for i in xrange(0,len(seq),size)]
+    return [seq[i:i+size] for i in range(0,len(seq),size)]
 
 
